
#----------------
# CODE FOR MCMC 
#----------------

prop.year.missing <- function(z.missing.index.subset,year.vector,current.mupsi1,current.prob.mat.2.n,
  current.z,current.area.j.beta,current.sim.alpha.distance,current.log.gamma,current.p.extinct) {
  year.missing=year.vector[min(z.missing.index.subset)]
  if(year.missing==1) {
    proposed.z.missing.vector=rbinom(length(z.missing.index.subset),1,current.mupsi1)
  }
  else {
    proposed.z.missing.vector=rbinom(length(z.missing.index.subset),1,current.prob.mat.2.n[z.missing.index.subset-nsite])
    }
  proposed.z=current.z[,year.missing]
  proposed.z[(z.missing.index.subset-(year.missing-1)*nsite)]=proposed.z.missing.vector
  proposed.area.occ=proposed.z*current.area.j.beta
  proposed.s.i.sq=(current.sim.alpha.distance%*%proposed.area.occ)^2
  proposed.p.colon=proposed.s.i.sq/(proposed.s.i.sq+exp(2*current.log.gamma))
  proposed.p.extinct.re=current.p.extinct*(1-proposed.p.colon) 
  proposed.prob=proposed.z*(1-proposed.p.extinct.re)+(1-proposed.z)*proposed.p.colon	

  log.prob.data.proposed=sum(current.z[,(year.missing+1)]*log(proposed.prob)+(1-current.z[,(year.missing+1)])*log(1-proposed.prob))
  log.prob.data.current=sum(current.z[,(year.missing+1)]*log(current.prob.mat.2.n[,year.missing])+
    (1-current.z[,(year.missing+1)])*log(1-current.prob.mat.2.n[,year.missing]))
    log.MH=log.prob.data.proposed-log.prob.data.current
    accept=decide(log.MH)
  if(accept) {
    current.z[z.missing.index.subset]=proposed.z.missing.vector
    current.prob.mat.2.n[,year.missing]=proposed.prob
    return(list(accept=accept,current.z=current.z,current.prob.mat.2.n=current.prob.mat.2.n))
  }
  else return(list(accept=accept))
}

ifm.missing.MCMC <- function(niter=1000,init,z.data, sim.distance, sim.area, sd.prop.mupsi1=0.1, sd.prop.mu=0.2, sd.prop.chi=0.2,sd.prop.gamma=0.2, sd.prop.beta=0.2, sd.prop.alpha=0.2,gamma.beta.bivnorm.high=FALSE, gamma.beta.bivnorm.low=FALSE,cov.prop.gamma.beta.high=matrix(c(2.22,0.58,0.58,0.17),nrow=2),cov.prop.gamma.beta.low=matrix(c(0.05,0.035,0.035,0.15),nrow=2), seed=round(runif(1,0,10000)), gamma.twice=FALSE,beta.twice=FALSE,alpha.twice=FALSE,nthin=1,nsite.subset=10,print.by=100) {
	
	iter.chain=seq(1,niter,by=nthin)
	iter.chain=sort(rep(iter.chain,nthin))
        iter.print=seq(1,niter,by=print.by)
        iter.print=sort(rep(iter.print,print.by))
	set.seed(seed)

        z.chain=NULL
       	mupsi1.chain=NULL
	log.mu.chain=NULL
	log.chi.chain=NULL
	log.gamma.chain=NULL
	log.beta.chain=NULL
	log.alpha.chain=NULL

	current.mupsi1=init$mupsi1
	current.log.mu=init$log.mu
	current.log.chi=init$log.chi
	current.log.gamma=init$log.gamma
	current.log.beta=init$log.beta
	current.log.alpha=init$log.alpha

        nyear=ncol(z.data)
        nsite=nrow(z.data)
	nobs=nsite*nyear
	nmissing=sum(is.na(z.data))
	year.vector=as.vector(col(z.data))
	z.index=c(1:nobs)
	z.missing.index=z.index[is.na(z.data)]
        z.missing.index.list=new.env()
        for(i in nyear:1) {
          temp=c(1:nobs)[is.na(z.data) & col(z.data)==i]
          assign(paste("z.missing.index.",i,sep=""),temp,envir=z.missing.index.list)
	}
        z.missing.index.list=as.list(z.missing.index.list)
        if(names(z.missing.index.list)[1]!="z.missing.index.1") stop("MISSING YEAR INDEX MIS-SORTED")

        z.missing.subset.vector=NULL
        n.missing.year=NULL
        
        for(i in 1:nyear) {
          temp=ceiling(length(z.missing.index.list[[i]])/nsite.subset)
          z.missing.subset.vector=c(z.missing.subset.vector,temp)
          n.missing.year=c(n.missing.year,length(z.missing.index.list[[i]]))
        }
        print(z.missing.subset.vector)
        print(n.missing.year)
        print(apply(is.na(z.data),2,sum))
        
	current.z=z.data
	current.z[is.na(z.data)]=init$z.missing
	current.z.1.nminus1=current.z[,1:(nyear-1)]
	current.z.2.n=current.z[,2:nyear]
	current.sim.alpha.distance=exp(-exp(current.log.alpha)*sim.distance)
	diag(current.sim.alpha.distance)=0
	current.area.j.beta=sim.area^exp(current.log.beta)
	current.area.occ.1.nminus1=current.z.1.nminus1*current.area.j.beta
	current.s.i.sq.1.nminus1=(current.sim.alpha.distance%*%current.area.occ.1.nminus1)^2
	current.p.colon.2.n=current.s.i.sq.1.nminus1/(current.s.i.sq.1.nminus1+exp(2*current.log.gamma))
	
	current.sim.area.chi=sim.area^exp(current.log.chi)
	current.p.extinct=exp(current.log.mu)/current.sim.area.chi
	current.p.extinct[current.p.extinct>1]=1
	current.p.extinct.re.2.n=current.p.extinct*(1-current.p.colon.2.n) 
	
	current.prob.mat.2.n=current.z.1.nminus1*(1-current.p.extinct.re.2.n)+(1-current.z.1.nminus1)*current.p.colon.2.n
	log.denominator.2.n=sum(current.z.2.n*log(current.prob.mat.2.n)+(1-current.z.2.n)*log(1-current.prob.mat.2.n))

	zero.m=rep(0.00001,nsite*(nyear-1))
	dim(zero.m)=c(nsite,(nyear-1))
	one.m=rep(0.99999,nsite*(nyear-1))
	dim(one.m)=c(nsite,(nyear-1))

        
	for (n in 1:niter) {
		#-------------------------
		#-----------MISSING DATA
		#-------------------------
          if(nmissing>0) {
          for(t in 1:(nyear-1)) {
            for(i in 1:z.missing.subset.vector[t]){
              if(i<z.missing.subset.vector[t]) temp.subset=z.missing.index.list[[t]][(nsite.subset*(i-1)+1):(nsite.subset*i)]
              else temp.subset=z.missing.index.list[[t]][(nsite.subset*(i-1)+1):length(z.missing.index.list[[t]])]
              temp=prop.year.missing(z.missing.index.subset=temp.subset,year.vector=year.vector,
                  current.mupsi1=current.mupsi1,current.prob.mat.2.n=current.prob.mat.2.n,
                  current.z=current.z,current.area.j.beta=current.area.j.beta,
                  current.sim.alpha.distance=current.sim.alpha.distance,current.log.gamma=current.log.gamma,
                  current.p.extinct=current.p.extinct)
                     if (temp$accept) {
                       current.z=temp$current.z
                       current.prob.mat.2.n=temp$current.prob.mat.2.n
                       }
                     }
            }
                
                current.z.1.nminus1=current.z[,1:(nyear-1)]
               	current.area.occ.1.nminus1=current.z.1.nminus1*current.area.j.beta
		current.s.i.sq.1.nminus1=(current.sim.alpha.distance%*%current.area.occ.1.nminus1)^2
		current.p.colon.2.n=current.s.i.sq.1.nminus1/(current.s.i.sq.1.nminus1+exp(2*current.log.gamma))
                
		#----------------------------
		# LAST YEAR MISSING SITES
		#----------------------------
		proposed.z.missing.vector.n=rbinom(length(z.missing.index.list[[nyear]]),1,current.prob.mat.2.n[z.missing.index.list[[nyear]]-nsite])
		current.z[z.missing.index.list[[nyear]]]=proposed.z.missing.vector.n
		current.z.2.n=current.z[,2:nyear]
        }
		#-----------------
		#-----------MUPSI1
		#-----------------
          if(sd.prop.mupsi1>0){
                log.denominator.1=sum(current.z[,1]*log(current.mupsi1)+(1-current.z[,1])*log(1-current.mupsi1))
                proposed.mupsi1=rnorm(1,current.mupsi1,sd=sd.prop.mupsi1)
		if(proposed.mupsi1 < 0.0001) {
			proposed.mupsi1=0.0002-proposed.mupsi1
			}
		if(proposed.mupsi1 >0.9999) {
			proposed.mupsi1=1.9998-proposed.mupsi1
			}
		log.numerator.1=sum(current.z[,1]*log(proposed.mupsi1)+(1-current.z[,1])*log(1-proposed.mupsi1))
		log.MH=log.numerator.1-log.denominator.1
		if(proposed.mupsi1<0) {
			log.mh=-100
			}
		accept=decide(log.MH)
		if(accept) {
			current.mupsi1=proposed.mupsi1
			}
              }
		#-----------------
		#-----------MU
                #-----------------
          if(sd.prop.mu>0){
               	log.denominator.2.n=sum(current.z.2.n*log(current.prob.mat.2.n)+(1-current.z.2.n)*log(1-current.prob.mat.2.n))
		proposed.log.mu=rnorm(1,current.log.mu,sd=sd.prop.mu) 
		if(proposed.log.mu<(-10)) {
			proposed.log.mu=-20-proposed.log.mu
			}
		if(proposed.log.mu>10) {
			proposed.log.mu=20-proposed.log.mu
			}
		proposed.p.extinct=exp(proposed.log.mu)/current.sim.area.chi
		proposed.p.extinct[proposed.p.extinct>1]=1
		proposed.p.extinct.re.2.n=proposed.p.extinct*(1-current.p.colon.2.n)
		proposed.prob.mat.2.n=current.z.1.nminus1*(1-proposed.p.extinct.re.2.n)+(1-current.z.1.nminus1)*current.p.colon.2.n
	
		log.numerator.2.n=sum(current.z.2.n*log(proposed.prob.mat.2.n)+(1-current.z.2.n)*log(1-proposed.prob.mat.2.n))
		log.MH=log.numerator.2.n-log.denominator.2.n
		accept=decide(log.MH)
		if (accept) {
			current.log.mu=proposed.log.mu
			current.p.extinct=proposed.p.extinct
			current.prob.mat.2.n=proposed.prob.mat.2.n
                        log.denominator.2.n=log.numerator.2.n
			}
              }
		#-----------------
		#-----------CHI
		#-----------------
          if(sd.prop.chi>0){
		proposed.log.chi=rnorm(1,current.log.chi,sd=sd.prop.chi)
		if(proposed.log.chi<(-10)) {
			proposed.log.chi=-20-proposed.log.chi
			}
		if(proposed.log.chi>10) {
			proposed.log.chi=20-proposed.log.chi
			}
		proposed.sim.area.chi=sim.area^exp(proposed.log.chi)
		proposed.p.extinct=exp(current.log.mu)/proposed.sim.area.chi
		proposed.p.extinct[proposed.p.extinct>1]=1
		proposed.p.extinct.re.2.n=proposed.p.extinct*(1-current.p.colon.2.n)
		proposed.prob.mat.2.n=current.z.1.nminus1*(1-proposed.p.extinct.re.2.n)+(1-current.z.1.nminus1)*current.p.colon.2.n

		log.numerator.2.n=sum(current.z.2.n*log(proposed.prob.mat.2.n)+(1-current.z.2.n)*log(1-proposed.prob.mat.2.n))
		log.MH=log.numerator.2.n-log.denominator.2.n
		accept=decide(log.MH)
		if (accept) {
                  	current.log.chi=proposed.log.chi
			current.sim.area.chi=proposed.sim.area.chi
			current.p.extinct=proposed.p.extinct
			current.prob.mat.2.n=proposed.prob.mat.2.n
                        log.denominator.2.n=log.numerator.2.n
			}
              }
		#-------------------
		# GAMMA AND BETA
		#-------------------
          if(gamma.beta.bivnorm.low==TRUE){
            stop("INFORMATIVE BETA; CODE NEEDS REVISION")
		temp=gamma.beta.bivnorm(cov.mat=cov.prop.gamma.beta.low,current.log.gamma=current.log.gamma,current.log.beta=current.log.beta,sim.area=sim.area,current.z.1.nminus1=current.z.1.nminus1,current.sim.alpha.distance=current.sim.alpha.distance,current.p.extinct=current.p.extinct,current.z.2.n=current.z.2.n,log.denominator.2.n=log.denominator.2.n,one.m=one.m,zero.m=zero.m)
                if(temp$accept) {
			current.log.beta=temp$current.log.beta
			current.log.gamma=temp$current.log.gamma
			current.area.j.beta=temp$current.area.j.beta
			current.area.occ.1.nminus1=temp$current.area.occ.1.nminus1
                        current.s.i.sq.1.nminus1=temp$current.s.i.sq.1.nminus1
                        current.p.colon.2.n=temp$current.p.colon.2.n
			current.prob.mat.2.n=temp$current.prob.mat.2.n
                        log.denominator.2.n=temp$log.denominator.2.n
			}
		}
          
          if(gamma.beta.bivnorm.high==TRUE){
            stop("INFORMATIVE BETA; CODE NEEDS REVISION")
		temp=gamma.beta.bivnorm(cov.mat=cov.prop.gamma.beta.high,current.log.gamma=current.log.gamma,current.log.beta=current.log.beta,sim.area=sim.area,current.z.1.nminus1=current.z.1.nminus1,current.sim.alpha.distance=current.sim.alpha.distance,current.p.extinct=current.p.extinct,current.z.2.n=current.z.2.n,log.denominator.2.n=log.denominator.2.n,one.m=one.m,zero.m=zero.m)
                if(temp$accept) {
			current.log.beta=temp$current.log.beta
			current.log.gamma=temp$current.log.gamma
			current.area.j.beta=temp$current.area.j.beta
			current.area.occ.1.nminus1=temp$current.area.occ.1.nminus1
                        current.s.i.sq.1.nminus1=temp$current.s.i.sq.1.nminus1
                        current.p.colon.2.n=temp$current.p.colon.2.n
			current.prob.mat.2.n=temp$current.prob.mat.2.n
                        log.denominator.2.n=temp$log.denominator.2.n
			}
		}
       
		#-----------------
		#-----------GAMMA
		#-----------------
          if(sd.prop.gamma>0){
		proposed.log.gamma=rnorm(1,current.log.gamma,sd=sd.prop.gamma)
		if(proposed.log.gamma<(-10)) {
			proposed.log.gamma=-20-proposed.log.gamma
			}
		if(proposed.log.gamma>10) {
			proposed.log.gamma=20-proposed.log.gamma
			}
		proposed.p.colon.2.n=current.s.i.sq.1.nminus1/(current.s.i.sq.1.nminus1+exp(2*proposed.log.gamma))
		proposed.p.extinct.re.2.n=current.p.extinct*(1-proposed.p.colon.2.n)
		proposed.prob.mat.2.n=current.z.1.nminus1*(1-proposed.p.extinct.re.2.n)+(1-current.z.1.nminus1)*proposed.p.colon.2.n
		log.numerator.2.n=sum(current.z.2.n*log(proposed.prob.mat.2.n)+(1-current.z.2.n)*log(1-proposed.prob.mat.2.n))
		log.MH=log.numerator.2.n-log.denominator.2.n
		accept=decide(log.MH)
		if(accept) {
			current.log.gamma=proposed.log.gamma
                        current.p.colon.2.n=proposed.p.colon.2.n
			current.prob.mat.2.n=proposed.prob.mat.2.n
                       	log.denominator.2.n=log.numerator.2.n
			}

                #SECOND UPDATE OF GAMMA
                if(gamma.twice==TRUE) {
	proposed.log.gamma=rnorm(1,current.log.gamma,sd=sd.prop.gamma)
		if(proposed.log.gamma<(-10)) {
			proposed.log.gamma=-20-proposed.log.gamma
			}
		if(proposed.log.gamma>10) {
			proposed.log.gamma=20-proposed.log.gamma
			}
		proposed.p.colon.2.n=current.s.i.sq.1.nminus1/(current.s.i.sq.1.nminus1+exp(2*proposed.log.gamma))
		proposed.p.extinct.re.2.n=current.p.extinct*(1-proposed.p.colon.2.n)
		proposed.prob.mat.2.n=current.z.1.nminus1*(1-proposed.p.extinct.re.2.n)+(1-current.z.1.nminus1)*proposed.p.colon.2.n
		log.numerator.2.n=sum(current.z.2.n*log(proposed.prob.mat.2.n)+(1-current.z.2.n)*log(1-proposed.prob.mat.2.n))
		log.MH=log.numerator.2.n-log.denominator.2.n
		accept=decide(log.MH)
		if(accept) {
			current.log.gamma=proposed.log.gamma
                        current.p.colon.2.n=proposed.p.colon.2.n
			current.prob.mat.2.n=proposed.prob.mat.2.n
                       	log.denominator.2.n=log.numerator.2.n
			}
                }
              }
   
	
		#-----------------
		#-----------BETA
		#-----------------
          if(sd.prop.beta>0) {
		proposed.log.beta=rnorm(1,current.log.beta,sd=sd.prop.beta)
		if(proposed.log.beta<(-10)) {
			proposed.log.beta=-20-proposed.log.beta
			}
		if(proposed.log.beta>0) {
			proposed.log.beta=-proposed.log.beta
			}
		proposed.area.j.beta=sim.area^exp(proposed.log.beta)
		proposed.area.occ.1.nminus1=current.z.1.nminus1*(proposed.area.j.beta)
		proposed.s.i.sq.1.nminus1=(current.sim.alpha.distance%*%proposed.area.occ.1.nminus1)^2
		proposed.p.colon.2.n=proposed.s.i.sq.1.nminus1/(proposed.s.i.sq.1.nminus1+exp(2*current.log.gamma))
		proposed.p.extinct.re.2.n=current.p.extinct*(1-proposed.p.colon.2.n) 
		proposed.prob.mat.2.n=current.z.1.nminus1*(1-proposed.p.extinct.re.2.n)+(1-current.z.1.nminus1)*proposed.p.colon.2.n
		log.numerator.2.n=sum(current.z.2.n*log(proposed.prob.mat.2.n)+(1-current.z.2.n)*log(1-proposed.prob.mat.2.n))
		log.MH=log.numerator.2.n-log.denominator.2.n
		accept=decide(log.MH)
		if(accept) {
			current.log.beta=proposed.log.beta
			current.area.j.beta=proposed.area.j.beta
			current.area.occ.1.nminus1=proposed.area.occ.1.nminus1
                        current.s.i.sq.1.nminus1=proposed.s.i.sq.1.nminus1
                        current.p.colon.2.n=proposed.p.colon.2.n
                       	current.prob.mat.2.n=proposed.prob.mat.2.n
			log.denominator.2.n=log.numerator.2.n
			}

                #SECOND UPDATE OF BETA
                if(beta.twice==TRUE) {
proposed.log.beta=rnorm(1,current.log.beta,sd=sd.prop.beta)
		if(proposed.log.beta<(-10)) {
			proposed.log.beta=-20-proposed.log.beta
			}
		if(proposed.log.beta>0) {
			proposed.log.beta=-proposed.log.beta
			}
		proposed.area.j.beta=sim.area^exp(proposed.log.beta)
		proposed.area.occ.1.nminus1=current.z.1.nminus1*(proposed.area.j.beta)
		proposed.s.i.sq.1.nminus1=(current.sim.alpha.distance%*%proposed.area.occ.1.nminus1)^2
		proposed.p.colon.2.n=proposed.s.i.sq.1.nminus1/(proposed.s.i.sq.1.nminus1+exp(2*current.log.gamma))
		proposed.p.extinct.re.2.n=current.p.extinct*(1-proposed.p.colon.2.n) 
		proposed.prob.mat.2.n=current.z.1.nminus1*(1-proposed.p.extinct.re.2.n)+(1-current.z.1.nminus1)*proposed.p.colon.2.n
		log.numerator.2.n=sum(current.z.2.n*log(proposed.prob.mat.2.n)+(1-current.z.2.n)*log(1-proposed.prob.mat.2.n))
		log.MH=log.numerator.2.n-log.denominator.2.n
		accept=decide(log.MH)
		if(accept) {
			current.log.beta=proposed.log.beta
			current.area.j.beta=proposed.area.j.beta
			current.area.occ.1.nminus1=proposed.area.occ.1.nminus1
                        current.s.i.sq.1.nminus1=proposed.s.i.sq.1.nminus1
                        current.p.colon.2.n=proposed.p.colon.2.n
                       	current.prob.mat.2.n=proposed.prob.mat.2.n
			log.denominator.2.n=log.numerator.2.n
			}
                }
              }
          
		#-----------------
		#-----------ALPHA
		#-----------------
          if(sd.prop.alpha>0) {
		proposed.log.alpha=rnorm(1,current.log.alpha,sd=sd.prop.alpha)
		if(proposed.log.alpha<(-10)) {
			proposed.log.alpha=-20-proposed.log.alpha
			}
		if(proposed.log.alpha>10) {
			proposed.log.alpha=20-proposed.log.alpha
			}
		proposed.sim.alpha.distance=exp(-exp(proposed.log.alpha)*sim.distance)
		diag(proposed.sim.alpha.distance)=0
		proposed.s.i.sq.1.nminus1=(proposed.sim.alpha.distance%*%current.area.occ.1.nminus1)^2
		proposed.p.colon.2.n=proposed.s.i.sq.1.nminus1/(proposed.s.i.sq.1.nminus1+exp(2*current.log.gamma))
		proposed.p.extinct.re.2.n=current.p.extinct*(1-proposed.p.colon.2.n) 
		proposed.prob.mat.2.n=current.z.1.nminus1*(1-proposed.p.extinct.re.2.n)+(1-current.z.1.nminus1)*proposed.p.colon.2.n		
		log.numerator.2.n=sum(current.z.2.n*log(proposed.prob.mat.2.n)+(1-current.z.2.n)*log(1-proposed.prob.mat.2.n))
		log.MH=log.numerator.2.n-log.denominator.2.n
		accept=decide(log.MH)
		if(accept) {
			current.log.alpha=proposed.log.alpha
			current.sim.alpha.distance=proposed.sim.alpha.distance
			current.s.i.sq.1.nminus1=proposed.s.i.sq.1.nminus1
			current.p.colon.2.n=proposed.p.colon.2.n
			current.prob.mat.2.n=proposed.prob.mat.2.n
                        log.denominator.2.n=log.numerator.2.n
			}

               #SECOND UPDATE OF ALPHA
                if(alpha.twice==TRUE) {
               	proposed.log.alpha=rnorm(1,current.log.alpha,sd=sd.prop.alpha)
		if(proposed.log.alpha<(-10)) {
			proposed.log.alpha=-20-proposed.log.alpha
			}
		if(proposed.log.alpha>10) {
			proposed.log.alpha=20-proposed.log.alpha
			}
		proposed.sim.alpha.distance=exp(-exp(proposed.log.alpha)*sim.distance)
		diag(proposed.sim.alpha.distance)=0
		proposed.s.i.sq.1.nminus1=(proposed.sim.alpha.distance%*%current.area.occ.1.nminus1)^2
		proposed.p.colon.2.n=proposed.s.i.sq.1.nminus1/(proposed.s.i.sq.1.nminus1+exp(2*current.log.gamma))
		proposed.p.extinct.re.2.n=current.p.extinct*(1-proposed.p.colon.2.n) 
		proposed.prob.mat.2.n=current.z.1.nminus1*(1-proposed.p.extinct.re.2.n)+(1-current.z.1.nminus1)*proposed.p.colon.2.n		
		log.numerator.2.n=sum(current.z.2.n*log(proposed.prob.mat.2.n)+(1-current.z.2.n)*log(1-proposed.prob.mat.2.n))
		log.MH=log.numerator.2.n-log.denominator.2.n
		accept=decide(log.MH)
		if(accept) {
			current.log.alpha=proposed.log.alpha
			current.sim.alpha.distance=proposed.sim.alpha.distance
			current.s.i.sq.1.nminus1=proposed.s.i.sq.1.nminus1
			current.p.colon.2.n=proposed.p.colon.2.n
			current.prob.mat.2.n=proposed.prob.mat.2.n
                        log.denominator.2.n=log.numerator.2.n
			}
              }
              }
	if(n==iter.chain[n]) {
                  z.chain=c(z.chain,current.z)
                  mupsi1.chain=c(mupsi1.chain,current.mupsi1)
                  log.mu.chain=c(log.mu.chain,current.log.mu)
                  log.chi.chain=c(log.chi.chain,current.log.chi)
                  log.gamma.chain=c(log.gamma.chain,current.log.gamma)
                  log.beta.chain=c(log.beta.chain,current.log.beta)
                  log.alpha.chain=c(log.alpha.chain,current.log.alpha)    
		  }
          
               if(n==iter.print[n]) {
                  print(n)
                }  
        }
                #----END ITERATIONS LOOP
       dim(z.chain)=c(nsite,nyear,length(log.alpha.chain))

        #FOR CONVENIENCE, CALCULATE OBSERVED DATA IN SAME MANNER AS IFM_ROBUST EVEN THOUGH IT IS NOT ESTIMATED.
        muz.chain=NULL
        muz.missing.chain=NULL
        n.extinct.chain=NULL
        n.colon.chain=NULL
        n.extinct.obs.chain=NULL
        n.colon.obs.chain=NULL
        n.occ.pairs.obs.chain=NULL

         z.index.list=new.env()
        for(i in (nyear-1):1){
          temp1=c(1:nsite)[!is.na(z.data[,i]) & !is.na(z.data[,(i+1)])]
          assign(paste("z.",i,".",(i+1),".index",sep=""),temp1,z.index.list)
        }
        z.index.list=as.list(z.index.list)
       if(names(z.index.list)[1]!="z.1.2.index") stop("Z INDEX MIS-SORTED")
        n.pairs.obs=NULL
        for (i in 1:(nyear-1)){
        n.pairs.obs=c(n.pairs.obs,length(z.index.list[[i]]))
        }
 
        for(t in 1:nyear) {
          muz.missing.chain=c(muz.missing.chain,apply(z.chain[(z.missing.index.list[[t]]-(nyear-1)*nsite),t,],2,mean))
          muz.chain=c(muz.chain,apply(z.chain[,t,],2,mean))
        }
        for(t in 1:(nyear-1)){
          n.extinct.mat.temp=(z.chain[,t,]==1 & z.chain[,(t+1),]==0)
          n.extinct.chain=c(n.extinct.chain,apply(n.extinct.mat.temp,2,sum))
          n.extinct.obs.chain=c(n.extinct.obs.chain,apply(n.extinct.mat.temp[z.index.list[[t]],],2,sum))

          n.colon.mat.temp=(z.chain[,t,]==0 & z.chain[,(t+1),]==1)
          n.colon.chain=c(n.colon.chain,apply(n.colon.mat.temp,2,sum))
          n.colon.obs.chain=c(n.colon.obs.chain,apply(n.colon.mat.temp[z.index.list[[t]],],2,sum))

          n.occ.pairs.obs.temp=apply(z.chain[z.index.list[[t]],t,],2,sum)
          n.occ.pairs.obs.chain=c(n.occ.pairs.obs.chain,n.occ.pairs.obs.temp)
        }
        muz.missing.chain=matrix(muz.missing.chain,nrow=nyear,byrow=TRUE)
        muz.chain=matrix(muz.chain,nrow=nyear,byrow=TRUE)
            
        n.extinct.chain=matrix(n.extinct.chain,nrow=(nyear-1),byrow=TRUE)
        prop.extinct.chain=n.extinct.chain/(nsite*muz.chain[1:(nyear-1),])
        
        n.colon.chain=matrix(n.colon.chain,nrow=(nyear-1),byrow=TRUE)
        prop.colon.chain=n.colon.chain/(nsite*(1-muz.chain[1:(nyear-1),]))
        
        n.extinct.obs.chain=matrix(n.extinct.obs.chain,nrow=(nyear-1),byrow=TRUE)
        n.colon.obs.chain=matrix(n.colon.obs.chain,nrow=(nyear-1),byrow=TRUE)
        n.occ.pairs.obs.chain=matrix(n.occ.pairs.obs.chain,nrow=(nyear-1),byrow=TRUE)

        prop.extinct.obs.chain=n.extinct.obs.chain/n.occ.pairs.obs.chain
        prop.colon.obs.chain=n.colon.obs.chain/(n.pairs.obs-n.occ.pairs.obs.chain)

        prop.extinct.missing.chain=(n.extinct.chain-n.extinct.obs.chain)/(nsite*muz.chain[1:(nyear-1),]-n.occ.pairs.obs.chain)
        prop.colon.missing.chain=(n.colon.chain-n.colon.obs.chain)/(nsite*(1-muz.chain[1:(nyear-1),])-(n.pairs.obs-n.occ.pairs.obs.chain))
     
        return(list(z.chain=z.chain,muz.missing.chain=muz.missing.chain,muz.chain=muz.chain,prop.extinct.obs.chain=prop.extinct.obs.chain,prop.extinct.missing.chain=prop.extinct.missing.chain,prop.extinct.chain=prop.extinct.chain,prop.colon.obs.chain=prop.colon.obs.chain,prop.colon.missing.chain=prop.colon.missing.chain,prop.colon.chain=prop.colon.chain,mupsi1.chain=mupsi1.chain,log.mu.chain=log.mu.chain,log.chi.chain=log.chi.chain,log.gamma.chain=log.gamma.chain,log.beta.chain=log.beta.chain,log.alpha.chain=log.alpha.chain))
}
